@@ -1,5 +1,5 @@
 # --- BEGIN COPYRIGHT BLOCK ---
-# Copyright (C) 2023 Red Hat, Inc.
+# Copyright (C) 2025 Red Hat, Inc.
 # All rights reserved.
 #
 # License: GPL (version 3 or any later version).
@@ -43,7 +43,7 @@ def _get_pw_policy(inst, targetdn, log, use_json=None):
         targetdn = 'cn=config'
         policydn = targetdn
         basedn = targetdn
-        attr_list.extend(['passwordisglobalpolicy', 'nsslapd-pwpolicy_local'])
+        attr_list.extend(['passwordisglobalpolicy', 'nsslapd-pwpolicy-local'])
         all_attrs = inst.config.get_attrs_vals_utf8(attr_list)
         attrs = {k: v for k, v in all_attrs.items() if len(v) > 0}
     else:
@@ -91,6 +91,7 @@ def list_policies(inst, basedn, log, args):
     else:
         result = ""
 
+    seen_dns = set()
     for targetdn in targetdns:
         # Verify target dn exists before getting started
         user_entry = Account(inst, targetdn)
@@ -104,13 +105,30 @@ def list_policies(inst, basedn, log, args):
         attr_list = list(pwp_manager.arg_to_attr.values())
 
         for pwp_entry in pwp_entries.list():
+            # Filter duplicates because subtree search on parent suffix also returns
+            # policies from sub suffixes
+            if pwp_entry.dn in seen_dns:
+                continue
+            seen_dns.add(pwp_entry.dn)
             # Sometimes, the cn value includes quotes (for example, after migration from pre-CLI version).
             # We need to strip them as python-ldap doesn't expect them
             dn_comps_str = pwp_entry.get_attr_val_utf8_l('cn').strip("\'").strip("\"")
-            dn_comps = ldap.dn.explode_dn(dn_comps_str)
-            dn_comps.pop(0)
-            entrydn = ",".join(dn_comps)
-            policy_type = _get_policy_type(inst, entrydn)
+            try:
+                dn_comps = ldap.dn.explode_dn(dn_comps_str)
+                dn_comps.pop(0)
+                entrydn = ",".join(dn_comps)
+                policy_type = _get_policy_type(inst, entrydn)
+            except ldap.DECODING_ERROR:
+                # This is some kind of custom password policy, the UI relies on
+                # on type being "Unknown policy type" in this.unknownPolicyType
+                policy_type = "Unknown policy type"
+                entrydn = "Unknown target"
+                if not args.json:
+                    # If not JSON then set a custom response, otherwise the
+                    # JSON result is sufficient
+                    result += "%s (%s)\n" % (pwp_entry.dn, policy_type.lower())
+                    continue
+
             all_attrs = pwp_entry.get_attrs_vals_utf8(attr_list)
             attrs = {k: v for k, v in all_attrs.items() if len(v) > 0}
             if args.json:
